==> dipex-4.20.0.tar.gz: dipex-4.20.0/integrations/SD_Lon/sdlon/date_utils.py <==
import logging
import re
from datetime import date
from datetime import datetime
from datetime import timedelta
from itertools import chain
from itertools import takewhile
from typing import OrderedDict
from typing import Union
from typing import Iterator
from typing import Optional
from typing import Tuple

from more_itertools import pairwise
from more_itertools import tabulate

from .sd_common import EmploymentStatus

# TODO: move constants elsewhere
# TODO: set back to "infinity" when MO can handle this
# MO_INFINITY: str = "infinity"

MO_INFINITY = None
SD_INFINITY: str = "9999-12-31"
DATE_REGEX_STR = "[0-9]{4}-(0[1-9]|1[0-2])-([0-2][0-9]|3[0-1])"

logger = logging.getLogger("sdImport")


def date_to_datetime(d: date) -> datetime:
    return datetime(d.year, d.month, d.day)


def format_date(date_time: datetime) -> str:
    return date_time.strftime("%Y-%m-%d").zfill(10)


def parse_datetime(date_str: str) -> datetime:
    return datetime.strptime(date_str, "%Y-%m-%d")


def datetime_to_sd_date(date_time: Union[datetime, date]) -> str:
    return date_time.strftime("%d.%m.%Y")


def _get_employment_from_date(employment: OrderedDict) -> datetime:
    """
    Get the latest date of all the dates in the payload from the SD
    GetEmployment endpoint (see https://redmine.magenta-aps.dk/issues/51898)

    Args:
        employment: the SD employment

    Returns: the maximum of all the date in the SD payload

    """
    MIN_DATE = format_date(datetime.min)

    employment_date = employment.get("EmploymentDate", MIN_DATE)
    employment_department_date = employment.get("EmploymentDepartment", dict()).get(
        "ActivationDate", MIN_DATE
    )
    employment_status_date = employment.get("EmploymentStatus", dict()).get(
        "ActivationDate", MIN_DATE
    )
    profession_date = employment.get("Profession", dict()).get(
        "ActivationDate", MIN_DATE
    )
    working_time_date = employment.get("WorkingTime", dict()).get(
        "ActivationDate", MIN_DATE
    )

    return max(
        parse_datetime(employment_date),
        parse_datetime(employment_department_date),
        parse_datetime(employment_status_date),
        parse_datetime(profession_date),
        parse_datetime(working_time_date),
    )


def get_employment_datetimes(employment: OrderedDict) -> Tuple[datetime, datetime]:
    """
    Get the "from" and "to" date from the SD employment

    Args:
        employment: The SD employment

    Returns:
        Tuple containing the "from" and "to" dates.
    """

    status = EmploymentStatus(employment["EmploymentStatus"]["EmploymentStatusCode"])

    if status in EmploymentStatus.let_go():
        datetime_from = parse_datetime(employment["EmploymentDate"])
        termination_date = str(
            sd_to_mo_termination_date(employment["EmploymentStatus"]["ActivationDate"])
        )
        datetime_to = parse_datetime(termination_date)
        return datetime_from, datetime_to

    datetime_from = _get_employment_from_date(employment)
    datetime_to = parse_datetime(employment["EmploymentStatus"]["DeactivationDate"])

    return datetime_from, datetime_to


# TODO: Create "MoValidity" and "SdValidity" classes based on the RA Models
#  "Validity" class and use these as input to the function below


def sd_to_mo_termination_date(sd_date: str) -> Optional[str]:
    """
    Convert SD termination date to MO termination date.

    Args:
        sd_date: SD termination date formatted as "YYYY-MM-DD"

    Returns:
        MO termination date formatted as "YYYY-MM-DD"
    """

    assert isinstance(sd_date, str)
    assert re.compile(DATE_REGEX_STR).match(sd_date)

    if sd_date == SD_INFINITY:
        return MO_INFINITY

    return sd_date


def to_midnight(dt: datetime) -> datetime:
    """Get previous midnight from datetime."""
    return dt.replace(hour=0, minute=0, second=0, microsecond=0)


def is_midnight(dt: datetime) -> bool:
    """Check if datetime is at midnight."""
    return dt == to_midnight(dt)


def gen_cut_dates(from_datetime: datetime, to_datetime: datetime) -> Iterator[datetime]:
    """Get iterator of cut-dates between from_datetime and to_datetime.

    Args:
        from_datetime: the start date
        to_datetime: the end date

    Yields:
        The from_datetime, then all intermediate midnight datetimes and the to_datetime.
    """
    assert from_datetime < to_datetime

    # Tabulate to infinite iterator of midnights starting after from_datetime
    def midnight_at_offset(offset: int) -> datetime:
        return to_midnight(from_datetime) + timedelta(days=offset)

    midnights = takewhile(
        lambda midnight: midnight < to_datetime, tabulate(midnight_at_offset, start=1)
    )
    return chain([from_datetime], midnights, [to_datetime])


def gen_date_intervals(
    from_datetime: datetime, to_datetime: datetime
) -> Iterator[Tuple[datetime, datetime]]:
    """
    Get iterator capable of generating a sequence of datetime pairs
    incrementing one day at a time. The latter date in a pair is
    advanced by exactly one day compared to the former date in the pair.

    Args:
        from_datetime: the start date
        to_datetime: the end date

    Yields:
        The next date pair in the sequence of pairs
    """
    return pairwise(gen_cut_dates(from_datetime, to_datetime))

==> dipex-4.25.0.tar.gz: dipex-4.25.0/integrations/SD_Lon/sdlon/date_utils.py <==
import logging
import re
from datetime import date
from datetime import datetime
from datetime import timedelta
from itertools import chain
from itertools import takewhile
from typing import OrderedDict
from typing import Union
from typing import Iterator
from typing import Optional
from typing import Tuple

from more_itertools import pairwise
from more_itertools import tabulate

from .sd_common import EmploymentStatus

# TODO: move constants elsewhere
# TODO: set back to "infinity" when MO can handle this
# MO_INFINITY: str = "infinity"

MO_INFINITY = None
SD_INFINITY: str = "9999-12-31"
DATE_REGEX_STR = "[0-9]{4}-(0[1-9]|1[0-2])-([0-2][0-9]|3[0-1])"

logger = logging.getLogger("sdImport")


def date_to_datetime(d: date) -> datetime:
    return datetime(d.year, d.month, d.day)


# TODO: rename
def format_date(d: datetime | date) -> str:
    return d.strftime("%Y-%m-%d").zfill(10)


def parse_datetime(date_str: str) -> datetime:
    return datetime.strptime(date_str, "%Y-%m-%d")


def datetime_to_sd_date(date_time: Union[datetime, date]) -> str:
    return date_time.strftime("%d.%m.%Y")


def _get_employment_from_date(employment: OrderedDict) -> datetime:
    """
    Get the latest date of all the dates in the payload from the SD
    GetEmployment endpoint (see https://redmine.magenta-aps.dk/issues/51898)

    Args:
        employment: the SD employment

    Returns: the maximum of all the date in the SD payload

    """
    MIN_DATE = format_date(datetime.min)

    employment_date = employment.get("EmploymentDate", MIN_DATE)
    employment_department_date = employment.get("EmploymentDepartment", dict()).get(
        "ActivationDate", MIN_DATE
    )
    employment_status_date = employment.get("EmploymentStatus", dict()).get(
        "ActivationDate", MIN_DATE
    )
    profession_date = employment.get("Profession", dict()).get(
        "ActivationDate", MIN_DATE
    )
    working_time_date = employment.get("WorkingTime", dict()).get(
        "ActivationDate", MIN_DATE
    )

    return max(
        parse_datetime(employment_date),
        parse_datetime(employment_department_date),
        parse_datetime(employment_status_date),
        parse_datetime(profession_date),
        parse_datetime(working_time_date),
    )


def get_employment_datetimes(employment: OrderedDict) -> Tuple[datetime, datetime]:
    """
    Get the "from" and "to" date from the SD employment

    Args:
        employment: The SD employment

    Returns:
        Tuple containing the "from" and "to" dates.
    """

    status = EmploymentStatus(employment["EmploymentStatus"]["EmploymentStatusCode"])

    if status in EmploymentStatus.let_go():
        datetime_from = parse_datetime(employment["EmploymentDate"])
        termination_date = str(
            sd_to_mo_termination_date(employment["EmploymentStatus"]["ActivationDate"])
        )
        datetime_to = parse_datetime(termination_date)
        return datetime_from, datetime_to

    datetime_from = _get_employment_from_date(employment)
    datetime_to = parse_datetime(employment["EmploymentStatus"]["DeactivationDate"])

    return datetime_from, datetime_to


# TODO: Create "MoValidity" and "SdValidity" classes based on the RA Models
#  "Validity" class and use these as input to the function below


def sd_to_mo_termination_date(sd_date: str) -> Optional[str]:
    """
    Convert SD termination date to MO termination date.

    Args:
        sd_date: SD termination date formatted as "YYYY-MM-DD"

    Returns:
        MO termination date formatted as "YYYY-MM-DD"
    """

    assert isinstance(sd_date, str)
    assert re.compile(DATE_REGEX_STR).match(sd_date)

    if sd_date == SD_INFINITY:
        return MO_INFINITY

    return sd_date


def to_midnight(dt: datetime) -> datetime:
    """Get previous midnight from datetime."""
    return dt.replace(hour=0, minute=0, second=0, microsecond=0)


def is_midnight(dt: datetime) -> bool:
    """Check if datetime is at midnight."""
    return dt == to_midnight(dt)


def gen_cut_dates(from_datetime: datetime, to_datetime: datetime) -> Iterator[datetime]:
    """Get iterator of cut-dates between from_datetime and to_datetime.

    Args:
        from_datetime: the start date
        to_datetime: the end date

    Yields:
        The from_datetime, then all intermediate midnight datetimes and the to_datetime.
    """
    assert from_datetime < to_datetime

    # Tabulate to infinite iterator of midnights starting after from_datetime
    def midnight_at_offset(offset: int) -> datetime:
        return to_midnight(from_datetime) + timedelta(days=offset)

    midnights = takewhile(
        lambda midnight: midnight < to_datetime, tabulate(midnight_at_offset, start=1)
    )
    return chain([from_datetime], midnights, [to_datetime])


def gen_date_intervals(
    from_datetime: datetime, to_datetime: datetime
) -> Iterator[Tuple[datetime, datetime]]:
    """
    Get iterator capable of generating a sequence of datetime pairs
    incrementing one day at a time. The latter date in a pair is
    advanced by exactly one day compared to the former date in the pair.

    Args:
        from_datetime: the start date
        to_datetime: the end date

    Yields:
        The next date pair in the sequence of pairs
    """
    return pairwise(gen_cut_dates(from_datetime, to_datetime))
